# **********************************************************************
# Copyright 2017 VMware, Inc.  All rights reserved. VMware Confidential.
# **********************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# **********************************************************************
import nsx_logging
import json

from cli.commands.cli_command_provider import CliCommandProvider
from cli.commands.lcp_common.hyperbus.client import get_hyperbus_cli_output
from cli.utils.error_code import ErrorCode as ec

lg = nsx_logging.getLogger("cli.commands.lcp_common.hyperbus.vif_lip")


def load_response_output(response):
    try:
        if response is None:
            return None
        output = json.loads(response)
    except (TypeError, ValueError) as err:
        lg.error("load response error: %s", err,
                 error_code=ec.INTERNAL_ERROR)
        return None
    return output


class GetVIFLIPTable(CliCommandProvider):
    def __call__(self, cmd_list):
        response = get_hyperbus_cli_output("vif-lip/table")
        output = load_response_output(response)
        if output is None:
            return "MSG_HOST_VIF_LIP_TABLE_FAILED"

        return "OK\n\n%s" % json.dumps(output)


class GetLIPByVIFID(CliCommandProvider):
    def __call__(self, cmd_list):
        vifID = cmd_list[3]
        response = get_hyperbus_cli_output("vif/lip", [vifID])
        output = load_response_output(response)
        if output is None:
            return "MSG_HOST_VIF_LIP_TABLE_FAILED"

        return "OK\n\n%s" % json.dumps(output)


class GetIPPool(CliCommandProvider):
    def __call__(self, cmd_list):
        response = get_hyperbus_cli_output("lip/pool")
        output = load_response_output(response)
        if output is None:
            return "MSG_HOST_VIF_LIP_TABLE_FAILED"

        return "OK\n\n%s" % json.dumps(output)
