# ***************************************************************************
# Copyright 2018 VMware, Inc.  All rights reserved. VMware Confidential.
# ***************************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# ***************************************************************************
import nsx_logging
import os
import signal
import sys
from subprocess import check_output, CalledProcessError

from cli.utils.command_utils import invoke_command
from cli.utils.error_code import ErrorCode as ec
from cli.utils.node_utils import get_node_type
from cli.utils.constants import NODE_TYPE_EDGE
from cli.utils.constants import NODE_TYPE_AUTONOMOUS_EDGE
from cli.utils.constants import NODE_TYPE_CONTROLLER
from cli.utils.constants import NODE_TYPE_MANAGER
from cli.utils.constants import NODE_TYPE_PUBLIC_CLOUD_GATEWAY
from cli.utils.constants import NODE_TYPE_HOST_RHELKVM

lg = nsx_logging.getLogger("cli.utils.signal_utils")

_SUDO = "/usr/bin/sudo"
_KILL = "/bin/kill"
_SIGHUP = "-SIGHUP"


def send_sighup(daemon_name):
    node_type = get_node_type()
    if node_type == NODE_TYPE_HOST_RHELKVM:
        pidof = "/usr/sbin/pidof"
    else:
        pidof = "/bin/pidof"

    try:
        pids = check_output([pidof, daemon_name])
    except CalledProcessError as ex:
        lg.exception("Failed to find daemon process id")
        return False

    pids = pids.decode("utf-8") if (sys.version_info >= (3,)
            and hasattr(pids, "decode")) else pids
    supported_types = [NODE_TYPE_EDGE, NODE_TYPE_AUTONOMOUS_EDGE,
                       NODE_TYPE_CONTROLLER, NODE_TYPE_MANAGER,
                       NODE_TYPE_PUBLIC_CLOUD_GATEWAY]

    for pid in pids.split():
        if any([sup_type in node_type for sup_type in supported_types]):
            cmd = [_SUDO, _KILL, _SIGHUP, pid]
            try:
                lg.debug("Sending SIGHUP to %s" % daemon_name)
                rc, _, err = invoke_command(cmd)
                if rc != 0:
                    lg.error("SIGHUP failed for pid: %s, return code: %s,\
                                error: %s" % (pid, rc, err),
                                error_code=ec.OPERATION_FAILED)
                    return False
            except Exception as ex:
                lg.exception("SIGHUP failed for pid: %s with exception: %s"
                                % (pid, ex), error_code=ec.INTERNAL_ERROR)
                return False
        else:
            try:
                os.kill(int(pid), signal.SIGHUP)
            except Exception as ex:
                lg.exception("SIGHUP failed for pid: %s with exception: %s"
                                % (pid, ex), error_code=ec.INTERNAL_ERROR)
                return False
    return True
