# Copyright (C) 2019 VMware, Inc. All rights reserved.
# -- VMware Confidential

from vmware.nsx.sha import sha_pb2

import sha.config
import time
from sha.contrib.action.abc_base import Action

from sha.contrib.action.utils.check import should_report

from sha.constants import REPORT_INTERVAL, SMALLEST_REPORT_INTERVAL_IF_CHANGE, MPA_HUB
from sha.core.channel.provider.mpa_provider import ReportMsg
from sha.config_monitor import ConfigMonitor


class Nareport(Action):
    _logger = sha.config.get_logger(__name__)
    pre_report_msg = None
    pre_report_time = None
    pre_reported_by_change = False  # indicate whether pre report is caused by data change

    @classmethod
    def populate_pb_msg(cls, **kwargs):
        report_msg = sha_pb2.ShaReportMsg()
        report_msg.oid.left, report_msg.oid.right = ConfigMonitor.host_id_low_high()
        connection_list = kwargs['connection_list']
        if connection_list:
            for conn in connection_list:
                status_msg = report_msg.status.add()
                status_msg.type = sha_pb2.ShaStatusMsg.ShaStatusType.Value('NODE_AGENT')
                status_msg.node_agent_status.vif_id = conn['id']
                conn_status = 'HEALTHY' if conn['stable'] is True else 'UNHEALTHY'
                status_msg.node_agent_status.hyperbus_connection_status = sha_pb2.Status.Value(conn_status)
        else:
            # if not retrieve any connection, report a msg only contains 'Type' to MP,
            # so that MP can clear previous outdated data
            status_msg = report_msg.status.add()
            status_msg.type = sha_pb2.ShaStatusMsg.ShaStatusType.Value('NODE_AGENT')
        return report_msg.SerializeToString()

    @classmethod
    def do(cls, hubs, *args):
        def kernel(**kwargs):
            cls._logger.debug('Nareport - signaling, connection_list:{}, profile:{}'
                              .format(kwargs['connection_list'], kwargs['profile']))

            pb_msg = cls.populate_pb_msg(**kwargs)
            profile = kwargs['profile']
            do_report, report_by_change = \
                should_report(cls.pre_report_msg, cls.pre_report_time, cls.pre_reported_by_change, pb_msg,
                              profile[REPORT_INTERVAL], profile[SMALLEST_REPORT_INTERVAL_IF_CHANGE])

            cls._logger.debug('Nareport - signaling, do_report:{}, report_by_change:{}'
                              .format(do_report, report_by_change))

            if do_report:
                cls._logger.debug('Nareport - reporting, connection_list:{}, profile:{}'
                                 .format(kwargs['connection_list'], kwargs['profile']))
                hubs[MPA_HUB].send(ReportMsg(b'', pb_msg, b'com.vmware.nsx.sha.ShaReportMsg'))
                cls.pre_report_time = time.monotonic()
                cls.pre_report_msg = pb_msg
                cls.pre_reported_by_change = report_by_change

        return kernel
