# Copyright (C) 2019 VMware, Inc. All rights reserved.
# -- VMware Confidential

import ctypes
import socket
import struct

import sha.config
from sha.contrib.monitor.uplinkstatus.esx.message import TLV, parse_notification

LIBC = ctypes.CDLL("libc.so.6")

PF_VMKLINK = 27
AF_VMKLINK = PF_VMKLINK


class KcpConnection(object):
    """
    Connection to KCP module in kernel.
    """
    KCP_SHA_ADDR_PATH = b'kcp_sha_vmklink'
    SOCKET_BUF_SIZE = 4096
    REMOTE_SOCK_PORT = 50001

    def __init__(self):
        self._logger = sha.config.get_logger(__name__)
        self._sock_kcp = None

    def connect_to_kcp(self):
        """ Connect esx dp netlink socket for kcp.
        """
        self._sock_kcp = socket.socket(PF_VMKLINK, socket.SOCK_DGRAM, 0)
        saddr = struct.pack('H108s', PF_VMKLINK,
                            KcpConnection.KCP_SHA_ADDR_PATH)
        if LIBC.connect(self._sock_kcp.fileno(), saddr, len(saddr)) != 0:
            self._logger.error("Cannot connect to kcp dataplane")
            self._sock_kcp = None
            return False
        else:
            self._logger.info("Connect to kcp dataplane")
            return True

    def is_connected(self):
        return self._sock_kcp is not None

    def recv_notification(self):
        if not self._sock_kcp:
            return None
        tlv = TLV.recv(self._sock_kcp)
        notifyTlv = TLV.unpack(tlv.value)
        notify = parse_notification(notifyTlv)
        self._logger.debug(notify)
        return notify

    def close(self):
        if self._sock_kcp:
            self._sock_kcp.close()
            self._sock_kcp = None